﻿/* slimtaskbar - memory poker
 * Copyright (C) 2010  ed <irc.rizon.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, please refer to the following URL:
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Diagnostics;

namespace slimtaskbar
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        [DllImport("kernel32.dll", SetLastError=true)]
        static extern IntPtr OpenProcess(IntPtr dwDesiredAccess, bool bInheritHandle, UInt32 dwProcessId);

        [DllImport("kernel32.dll", SetLastError = true)]
        static extern bool ReadProcessMemory(IntPtr hProcess, IntPtr lpBaseAddress, [In, Out] byte[] lpBuffer, UInt32 nSize, out IntPtr lpNumberOfBytesRead);

        [DllImport("kernel32.dll", SetLastError = true)]
        static extern bool WriteProcessMemory(IntPtr hProcess, IntPtr lpBaseAddress, [In, Out] byte[] lpBuffer, UInt32 nSize, out IntPtr lpNumberOfBytesWritten);

        [DllImport("kernel32.dll", SetLastError = true)]
        static extern bool CloseHandle(IntPtr hObject);

        Config config;
        TextBox[] ui;
        
        void run()
        {
            string[] soffsets = _adr.Text.Split(' ');
            int[] offsets = new int[soffsets.Length];
            for (int a = 0; a < offsets.Length; a++)
            {
                offsets[a] = Convert.ToInt32(soffsets[a], 16);
            }
            IntPtr baseAddress = IntPtr.Zero;
            Process proc = Process.GetProcessesByName(_proc.Text)[0];
            foreach (ProcessModule pm in proc.Modules)
            {
                if (pm.ModuleName.ToLower() == _mod.Text)
                {
                    baseAddress = pm.BaseAddress;
                }
            }
            if (baseAddress == IntPtr.Zero)
            {
                error("Failed to find base address of " + _mod.Text);
            }
            IntPtr yea;
            byte[] buffer = new byte[4];
            int[] values = new int[offsets.Length];
            IntPtr access = (IntPtr)0x1F0FFF; //EVERYTHING.
            IntPtr hnd = OpenProcess(access, false, (UInt32)proc.Id);
            for (int a = 0; a < offsets.Length; a++)
            {
                IntPtr ptr = baseAddress + offsets[a];
                if (!ReadProcessMemory(hnd, ptr, buffer, 4, out yea))
                {
                    throw new System.ComponentModel.Win32Exception();
                }
                values[a] = BitConverter.ToInt32(buffer, 0);
                if (values[a] < 1 || values[a] > 250 ||
                    (a > 0 && values[a] != values[0]))
                {
                    error("Safety check failed:\r\nValue " + (a + 1) + " mismatch (" + values[a] + ")");
                }
            }
            int val = Convert.ToInt32(_val.Text);
            IntPtr target = baseAddress + offsets[0];
            buffer = BitConverter.GetBytes((Int32)val);
            if (!WriteProcessMemory(hnd, target, buffer, 4, out yea))
            {
                throw new System.ComponentModel.Win32Exception();
            }
            error("");
        }

        void error(string msg)
        {
            if (!string.IsNullOrEmpty(msg))
            {
                MessageBox.Show(msg, "Critical error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            System.Diagnostics.Process.GetCurrentProcess().Kill();
        }

        private void _run_Click(object sender, EventArgs e)
        {
            run();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            Timer t = new Timer();
            t.Tick += new EventHandler(t_Tick);
            t.Interval = 10;
            t.Start();
        }

        void t_Tick(object sender, EventArgs e)
        {
            ((Timer)sender).Stop();
            ui = new TextBox[] { _proc, _mod, _adr, _val };
            string str = Application.ExecutablePath.Substring(Application.StartupPath.Length + 1);
            str = str.Substring(0, str.LastIndexOf('.'));
            config = new Config(str);
            config.loadFromFile();
            int i = 0;
            foreach (Profile p in config.profile)
            {
                _pro.Items.Add(p.name);
                if (p.name == config.activeProfile)
                {
                    i = _pro.Items.Count;
                }
            }
            _pro.SelectedIndex = i - 1;
            config.loadProfile(_pro.Items[_pro.SelectedIndex].ToString(), ui);
            _auto.Checked = config.autorun;
            if (config.autorun) run();
        }

        private void _sto_Click(object sender, EventArgs e)
        {
            config.autorun = _auto.Checked;
            config.saveProfile(_pro.Items[_pro.SelectedIndex].ToString(), ui);
        }

        private void _del_Click(object sender, EventArgs e)
        {
            int i = _pro.SelectedIndex;
            string s = _pro.Items[i].ToString();
            config.deleteProfile(s);
            _pro.Items.RemoveAt(i);
            if (i >= _pro.Items.Count)
            {
                i = _pro.Items.Count - 1;
            }
            _pro.SelectedIndex = i;
        }

        private void _lod_Click(object sender, EventArgs e)
        {
            config.loadProfile(_pro.Items[_pro.SelectedIndex].ToString(), ui);
        }
    }
}
