﻿/* slimtaskbar - memory poker
 * Copyright (C) 2010  ed <irc.rizon.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, please refer to the following URL:
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace slimtaskbar
{
    class Profile
    {
        public string name;
        public string proc;
        public string mod;
        public string adr;
        public string val;

        public Profile(string n, string p, string m, string a, string v)
        {
            name = n;
            proc = p;
            mod = m;
            adr = a;
            val = v;
        }
    }

    class Config
    {
        string prefix;
        public bool autorun;
        public string activeProfile;
        public List<Profile> profile;

        public Config(string appname)
        {
            profile = new List<Profile>();
            prefix = appname;
        }

        public void loadFromFile()
        {
            string[] raw;
            string fn = prefix + ".ini";
            if (System.IO.File.Exists(fn))
            {
                raw = System.IO.File.ReadAllLines(prefix + ".ini", Encoding.UTF8);
            }
            else
            {
                raw = new string[]
                {
                    "0 Windows 7 SP1 (6.1.7601)",
                    "",
                    "Windows 7 SP1 (6.1.7601)",
                    "explorer",
                    "explorer.exe",
                    "E886C E8CE0 E96B4 E96C8",
                    "36"
                };
            }
            if (raw.Length < 2) return;
            autorun = raw[0][0] == '1';
            activeProfile = raw[0].Substring(2);
            int fileptr = 1;
            while (true)
            {
                if (raw.Length > fileptr + 5)
                {
                    profile.Add(new Profile(
                        raw[++fileptr],
                        raw[++fileptr],
                        raw[++fileptr],
                        raw[++fileptr],
                        raw[++fileptr]
                    ));
                    fileptr++;
                }
                else break;
            }
        }

        public void loadProfile(string profileName, System.Windows.Forms.TextBox[] fields)
        {
            activeProfile = profileName;
            for (int a = 0; a < profile.Count; a++)
            {
                if (profile[a].name == activeProfile)
                {
                    fields[0].Text = profile[a].proc;
                    fields[1].Text = profile[a].mod;
                    fields[2].Text = profile[a].adr;
                    fields[3].Text = profile[a].val;
                    break;
                }
            }
        }

        public void saveProfile(string profileName, System.Windows.Forms.TextBox[] fields)
        {
            activeProfile = profileName;
            for (int a = 0; a < profile.Count; a++)
            {
                if (profile[a].name == activeProfile)
                {
                    profile.RemoveAt(a);
                    a--;
                }
            }
            profile.Add(new Profile(
                profileName,
                fields[0].Text,
                fields[1].Text,
                fields[2].Text,
                fields[3].Text
            ));
            StringBuilder sb = new StringBuilder();
            sb.AppendLine((autorun ? "1 " : "0 ") + activeProfile);
            sb.AppendLine();
            foreach (Profile p in profile)
            {
                sb.AppendLine(p.name);
                sb.AppendLine(p.proc);
                sb.AppendLine(p.mod);
                sb.AppendLine(p.adr);
                sb.AppendLine(p.val);
                sb.AppendLine();
            }
            System.IO.File.WriteAllText(prefix + ".ini", sb.ToString(), Encoding.UTF8);
        }

        public void deleteProfile(string profileName)
        {
            for (int a = 0; a < profile.Count; a++)
            {
                if (profile[a].name == profileName)
                {
                    profile.RemoveAt(a);
                    a--;
                }
            }
        }
    }
}
